// game.h

// Globally disable these warnings
#pragma warning (disable : 4018)
#pragma warning (disable : 4312)

#include <windows.h>
#include <vector>
#include <string>
#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <fmod.h> 
#include <math.h>
#include <time.h>

struct newPlayerInfo_t
{
	long x;
	long y;
	long z;
	long sector;
	long ang;
};

using namespace std;

#define GAME_BUILD "Game build: Build 17.March 06 2005\n"

#define BLOODSTATE_UNKNOWN  0 // for whatever reason state is invalid
#define BLOODSTATE_LOADING  1
#define BLOODSTATE_GAME     2
#define BLOODSTATE_MENU     3

#define PLAYER_WALKSPEED     5
#define PLAYER_RUNSPEED     10

#define PLAYER_WEAPON_PITCHFORK 0
#define PLAYER_WEAPON_FLAREGUN  1
#define PLAYER_WEAPON_STAFF     8


// From the Duke3d Source code
#define NORMALTURN   15	
#define PREAMBLETURN 5  	
#define NORMALKEYMOVE 40 	
#define MAXVEL       ((NORMALKEYMOVE*2)+10)	
#define MAXSVEL      ((NORMALKEYMOVE*2)+10)	
#define MAXANGVEL    127

struct bloodWeapon_t
{
	string weaponName;
	int    weaponID;
	int    ammo;
	int    idlePic;
	bloodWeapon_t::bloodWeapon_t(string weaponName, int weaponID, int ammo,
								 int idlePic)
	{
		this->weaponName = weaponName;
		this->weaponID   = weaponID;
		this->ammo       = ammo;
		this->idlePic    = idlePic;
	}
};

typedef char* buffer32;

struct bloodPlayer_t
{
	int  health;
	int  armor;

	string name;

	long   posx;
	long   posy;
	long   posz;

	long   avel;

	long   posxv;
	long   posxy;

	long   svel;
	long   vel;

	short  ang;
	short  fakangle;

	short  sectorNum;

	int  serverID;   // id on the server, given at Login time.  Used only for player 0

	int  thisWeapon;

	vector<bloodWeapon_t *> playerweapons;
};

// Includes
#include "fileManager.h"
extern fileManager_t   *fileManager;

#include "threads.h"

struct monster_t
{
	int      startPic;
	int      endPic;
	thread_t *mobAI;
};

struct monsterEnt_t
{
	int       mobID;
	int       aiNum;
	monster_t *thisMob;
};

typedef struct
{
	long x, y, z;
	short cstat, picnum;
	signed char shade;
	unsigned char pal, clipdist, filler;
	unsigned char xrepeat, yrepeat;
	signed char xoffset, yoffset;
	short sectnum, statnum;
	short ang, owner, xvel, yvel, zvel;
	short lotag, hitag, extra;
} spritetype;

class Game
{
	public:
		Game();
		int     RunFrame(int totalclock);
		void    g_LoadLevel(char *board);
		boolean g_PlayerInGame(void);
		void    DisplayLevel(void);
		void    Error(char *err, ...);
		void    Shutdown(void);
		void    Register_Weapon(char *weaponScript);
		void    RegisterAI(int startPic, int endPic, char *script_name);
		void    ModifySprite(int spriteNum, spritetype *thisSprite);
		void    StartAI(void);
		void    ShutdownAI(void);
		void    SuspendAI(void);
		void	ReRunAI(void);
		void    ShotWeapon(int damage);

		int     thisWeaponPic;
		int     thisWeaponY;
		int     thisWeaponX;
		int     weaponInUse;

		// Some weapons have two parts to them
		int     thisWeaponPic2;
		int     thisWeaponY2;
		int     thisWeaponX2;
		int     weaponInUse2;

		void    SetPlayerWeapon(int weaponNum);

		int     SetWeaponPic(int picNum)
		{
			this->thisWeaponPic = picNum;
		}
		
		vector<thread_t *>     playerWeapons;
		vector<monster_t *>    registeredAI;
		vector<monsterEnt_t *> worldMobs;

		boolean scripts_loaded;

		boolean exit_game;

		thread_t *gameThread;

		void LoadingPlague(void);
		void EndLoadingPlague(void);

		char *thisBoard;

		boolean loadingplague;

		vector<int> removeSprites;
	private:
		void    init_system(void);
		void    DrawHudNumber(int number, long startfont, int shade, int x, int y);
		void    DrawHUD(void);
		void    P_HandleMovement(void);
		void    Level_MakeActors(void);
		void    ThreadTest(void);
		void    RemoveEffectors(void);
		void    ShutdownWeaponThreads(void);
		int initframe;
		boolean ingame;

		vector<bloodPlayer_t *> gamePlayers;
	protected:
		int gamestate;
};

#include "guiManager.h"
#include "game_shared.h"
#include "game_menu.h"
#include "keyboard.h"
#include "bloodTiles.h"
#include "soundManager.h"
#include "buildMath.h"
#include "bloodActor.h"


#ifndef min
#define min(a, b)  (((a) < (b)) ? (a) : (b))
#endif

#ifndef max
#define max(a, b)  (((a) > (b)) ? (a) : (b))
#endif

struct build_func_t
{
	void (*nextpage)(void);
	boolean (*build_KeyDown)(byte key);
	void (*Load_Pallete)(char *filename);
	void (*rotatesprite)(long sx, long sy, long z, short a, short picnum,
                  signed char dashade, char dapalnum, char dastat,
                  long cx1, long cy1, long cx2, long cy2);
	void (*clearview)(long dacol);
	int  (*loadboard)(char *filename, long *daposx, long *daposy,
              long *daposz, short *daang, short *dacursectnum);
	void (*SDL_Quit)(void);
	int  (*Decrypt_Map)(char *in, char *out);
	void (*displayLevel)(long posx, long posy, long posz, long ang,short snum,long smoothratio);
	long (*scale)(long i1, long i2, long i3);
	newPlayerInfo_t (*P_Movecode)(short ang, long fvel, long svel);
	int (*clipmove) (long *x, long *y, long *z, short *sectnum,
              long xvect, long yvect, long walldist, long ceildist,
              long flordist, unsigned long cliptype);

	int (*mulscale) (int input1, int input2, int input3);
	int (*pushmove)(long *x, long *y, long *z, short *sectnum,
             long walldist, long ceildist, long flordist,
             unsigned long cliptype);
	int (*ftotalclock)(void);
	void (*Set_Brightness)(int brightness);
	void (*getzrange)(long x, long y, long z, short sectnum,
			long *ceilz, long *ceilhit, long *florz, long *florhit,
			long walldist, unsigned long cliptype);
	spritetype *(*Game_GetSprite)(int i);
	void (*Justin_SetSky)(long pScale, short newpskyoff[256]);
	void (*build_ClearKey)(byte key);
	short sintable[2048];
};

#define CLIPMASK0 (((1L)<<16)+1L)

// Game Managers
extern Game            *thisGame;
extern guiManager      *guimanager;
extern fileManager_t   *fileManager;
extern gameMenu_t      *menuManager;
extern soundManager_t  *soundManager;
extern actorManager_t  *actorManager;
extern build_func_t    *build_func;