//-------------------------------------------------------------------------
/*
Copyright (C) 2005 - Justin Marshall

This file is part of the blood reconstruction project.

WinBlood is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

PORTIONS OF CODE FROM DUKE NUKEM 3D SOURCE CODE
*/
//-------------------------------------------------------------------------

// network.cpp

#include <initguid.h>

#include "game.h"
#include "packets.h"

kenNet_t::kenNet_t()
{
	this->net_Initilized = false;
}


/*
===================================================================================================
HRESULT WINAPI DirectPlayMessageHandler(void *pvUserContext, DWORD dwMessageId, void *pMsgBuffer)

Handles DirectPlay messages
===================================================================================================
*/

HRESULT WINAPI DirectPlayMessageHandler(void *pvUserContext, DWORD dwMessageId, void *pMsgBuffer)
{
	switch( dwMessageId )
	{
		case DPN_MSGID_RECEIVE:
		{
			PDPNMSG_RECEIVE     pMsg;

			pMsg = (PDPNMSG_RECEIVE) pMsgBuffer;
			//process message

			break;
		}
	}
	return(DPN_OK);
}

/*
========================================
void kenNet_t::netinit (void)

Info:
Starts up DirectPlay
========================================
*/

void kenNet_t::netinit (void)
{
	WSADATA ws;
	HRESULT hr;

	gamePort = 5123;

	if(net_Initilized == true)
		return;
	CoInitialize(NULL);

	hr = CoCreateInstance( CLSID_DirectPlay8Peer, NULL, 
                          CLSCTX_INPROC_SERVER,
                          IID_IDirectPlay8Peer, 
                          (LPVOID*) &pDPlay );

	pDPlay->Initialize(NULL, DirectPlayMessageHandler, 0 );
	net_Initilized = true;
	g_pDeviceAddress = NULL;
	g_pDeviceRemoteAddress = NULL;
}

/*
====================================
void kenNet_t::netServerInit(void)
====================================
*/

void kenNet_t::netServerInit(void)
{
	HRESULT hr;
	ZeroMemory(&dpAppDesc, sizeof(DPN_APPLICATION_DESC));
    dpAppDesc.dwSize          = sizeof(DPN_APPLICATION_DESC);
	dpAppDesc.dwFlags         = DPNSESSION_CLIENT_SERVER | DPNSESSION_NODPNSVR;
    dpAppDesc.guidApplication = BLOODGAME_GUID;
	dpAppDesc.dwMaxPlayers    = 0;

	
	// Create our IDirectPlay8Address Device Address
    hr = CoCreateInstance( CLSID_DirectPlay8Address, NULL,
                                       CLSCTX_INPROC_SERVER,
                                       IID_IDirectPlay8Address,
                                       (LPVOID*) &g_pDeviceAddress ); 
    // Set the SP for our Device Address
    hr = g_pDeviceAddress->SetSP(&CLSID_DP8SP_TCPIP );

	g_pDeviceAddress->AddComponent(DPNA_KEY_PORT,&gamePort,sizeof(DWORD),DPNA_DATATYPE_DWORD);

    // You are now ready to host the application.
    hr = pDPlay->Host( &dpAppDesc,          // AppDesc
                      &g_pDeviceAddress, 1,// Device Address
                      NULL, NULL,          // Reserved
                      NULL,                // Player Context
                      0 );                 // dwFlags
}

/*
============================================
boolean kenNet_t::netConnect(char *ipAddress)
============================================
*/

boolean kenNet_t::netConnect(char *ipAddress)
{
	HRESULT						hr;
	DPN_APPLICATION_DESC        dpnAppDesc;
	WCHAR           strBuffer[128] = {0};
	DXUtil_ConvertGenericStringToWideCch( strBuffer, ipAddress, 128 );

	CoCreateInstance(CLSID_DirectPlay8Address, NULL,
                     CLSCTX_INPROC_SERVER,
                     IID_IDirectPlay8Address,
                     (LPVOID*) &g_pDeviceRemoteAddress ); 


	g_pDeviceRemoteAddress->AddComponent(DPNA_KEY_PORT,&gamePort,sizeof(DWORD),DPNA_DATATYPE_DWORD);

	ZeroMemory(&dpnAppDesc, sizeof(DPN_APPLICATION_DESC));
    dpnAppDesc.dwSize          = sizeof(DPN_APPLICATION_DESC);
    dpnAppDesc.guidApplication = BLOODGAME_GUID;

	hr = g_pDeviceRemoteAddress->AddComponent(DPNA_KEY_HOSTNAME, strBuffer,
													2*((UINT)wcslen(strBuffer) + 1), //bytes

													DPNA_DATATYPE_STRING );


	hr = pDPlay->Connect(&dpnAppDesc,    // Application Description
                        g_pDeviceRemoteAddress,       // Host Address
                        g_pDeviceAddress,   
                        NULL,  
                        NULL,
                        NULL, 0, 
                        NULL, 
                        NULL,
                        NULL, 
                        DPNCONNECT_SYNC);

	if( FAILED( hr))
		return false;
	else
		return true;
}